/* !! This file was automatically generated by NEWVERS !! */

/* for logfiles, etc. */
char ePerl_Version[] =
    "2.2b9 (28-06-1997)";

/* interactive 'hello' string to identify us to the user */
char ePerl_Hello[] = 
    "This is ePerl Version 2.2b9 (28-06-1997)";

/* a GNU --version output */
char ePerl_GNUVersion[] =
    "ePerl Version 2.2b9";

/* a UNIX what(1) id string */
char ePerl_WhatID[] =
    "@(#)ePerl Version 2.2b9 (28-06-1997)";

/* a RCS ident(1) id string */
char ePerl_RCSIdentID[] =
    "$Id: ePerl 2.2b9 28-06-1997 $";

/* a WWW id string */
char ePerl_WebID[] =
    "ePerl/2.2b9";

/* a plain id string */
char ePerl_PlainID[] =
    "2.2b9";

