<html>
<head>
<title>demo.pp</title>
</head>
<body>
<blockquote>
<h1>demo.pp</h1>
<p>
<!-- 
##
##  ePerl Demo: usage of the ePerl preprocessor
##  Copyright (c) 1996,1997 Ralf S. Engelschall, All Rights Reserved. 
##
##  This just demonstrates how one can use the
##  ePerl preprocessor to inline an external file.
##
-->

Here comes the ePerl README file, retrieved via ePerl-#include:

<pre>
#include http://www.engelschall.com/sw/eperl/distrib/eperl-SNAP/README
</pre>

</blockquote>
</body>
</html>
