<html>
<head>
<title>demo.html</title>
</head>
<body>
<blockquote>
<h1>demo.html</h1>
<p>
<!-- 
##
##  ePerl Demo: usage of the HTML-Stream package from within ePerl
##  Copyright (c) 1996,1997 Ralf S. Engelschall, All Rights Reserved. 
##
##  This just demonstrates how one can use the
##  HTML-Stream package from within ePerl.
##
-->

<?
use HTML::Stream;
$HTML = new HTML::Stream \*STDOUT;
!>

First, a programmed hyperlink to
<? $HTML->
   A(HREF=>"http://www.engelschall.com/sw/eperl/")->
   t("the ePerl webarea")->
   _A; !> .

<p>
Second, just 
<? $HTML->t("plain text with umlaut characters: äöüÄÖÜß\n" ); !>.<br>
These are automatically converted to HTML entities.

</blockquote>
</body>
</html>
