<html>
<head>
<title>demo.net</title>
</head>
<body>
<blockquote>
<h1>demo.net</h1>
<p>
<!-- 
##
##  ePerl Demo: usage of the Net package from within ePerl
##  Copyright (c) 1996,1997 Ralf S. Engelschall, All Rights Reserved. 
##
##  This just demonstrates how one can use the
##  Net (libnet) package from within ePerl.
##
-->

Here comes the ePerl README file, retrieved via libnet/FTP:

<pre>
<?
use Net::FTP;

$tmpfile = "/tmp/demo.net.tmp.$$";

$ftp = Net::FTP->new("ftp.engelschall.com");
$ftp->login("anonymous", "demo.net.phtml\@foreigner.com");
$ftp->cwd("/sw");
$ftp->get("eperl-2.1.2.readme", $tmpfile);
$ftp->quit;

open(FP, "<$tmpfile");
print <FP>;
close(FP);

unlink($tmpfile);
!>
</pre>

</blockquote>
</body>
</html>  
