<html>
<head>
<title>demo.env</title>
</head>
<body>
<blockquote>
<h1>demo.env</h1>
<p>
<!-- 
##
##  ePerl Demo: printing of the CGI/ePerl environment
##  Copyright (c) 1996,1997 Ralf S. Engelschall, All Rights Reserved. 
##
##  This just prints out the CGI environment provided
##  by the Webserver as a sorted list consisting of
##  key/value pairs.
##
-->

Our CGI/ePerl environment currently looks like this:

<pre>
<? foreach $key (sort(keys(%ENV))) { 
       print "$key=$ENV{$key}\n"; 
   }
!>
</pre>

</blockquote>
</body>
</html>
