/* README.c -- automatically generated by asc2c */

char *ePerl_README = \
"        ____           _ \n"\
"    ___|  _ \\ ___ _ __| |\n"\
"   / _ \\ |_) / _ \\ '__| |\n"\
"  |  __/  __/  __/ |  | |\n"\
"   \\___|_|   \\___|_|  |_|\n"\
"                         \n"\
"  ePerl -- Embedded Perl 5 Language\n"\
"\n"\
"  Version 2.2b9 (28-06-1997)\n"\
"\n"\
"  ePerl interprets an ASCII file bristled with Perl 5 program statements by\n"\
"  evaluating the Perl 5 code while passing through the plain ASCII data. It\n"\
"  can operate in various ways: As a stand-alone Unix filter or integrated Perl\n"\
"  5 module for general file generation tasks and as a powerful Webserver\n"\
"  scripting language for dynamic HTML page programming. \n"\
"\n"\
"  The documentation and latest release can be found on\n"\
"  http://www.engelschall.com/sw/eperl/\n"\
"\n"\
"  Copyright (c) 1996-1997 Ralf S. Engelschall, All rights reserved.\n"\
"\n"\
"  This program is free software; it may be redistributed and/or modified only\n"\
"  under the terms of either the Artistic License or the GNU General Public\n"\
"  License, which may be found in the ePerl source distribution.  Look at the\n"\
"  files ARTISTIC and COPYING or run ``eperl -l'' to receive a built-in copy of\n"\
"  both license files.\n"\
"\n"\
"  This program is distributed in the hope that it will be useful, but WITHOUT\n"\
"  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or\n"\
"  FITNESS FOR A PARTICULAR PURPOSE.  See either the Artistic License or the\n"\
"  GNU General Public License for more details.\n"\
"\n"\
"                              Ralf S. Engelschall\n"\
"                              rse@engelschall.com\n"\
"                              www.engelschall.com\n"\
"\n"\
;

/*EOF*/
